"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const react_1 = require("react");
const error_1 = require("./error");
const store_1 = require("./store");
function Consumer(props) {
    const store = useStore(props.of);
    return props.children(store);
}
exports.Consumer = Consumer;
function useStore(s, deps) {
    const name = s.displayName || s.name;
    react_1.useDebugValue(name);
    const Context = store_1.getStoreContext(s);
    const container = react_1.useContext(Context);
    const [state, setState] = react_1.useState(container.state);
    const depsRef = react_1.useRef([]);
    react_1.useEffect(() => {
        const subscriber = () => {
            if (!deps) {
                setState(container.state);
            }
            else {
                const oldDeps = depsRef.current;
                const newDeps = deps(container.state);
                if (compare(oldDeps, newDeps)) {
                    setState(container.state);
                }
                depsRef.current = newDeps;
            }
        };
        container.subscribers.add(subscriber);
        return () => {
            container.subscribers.delete(subscriber);
        };
    }, []);
    if (state === store_1.defaultStoreValue) {
        throw new error_1.NoStoreError(`No store context of "${name}" found. And "${name}" doesn't have defaultValue. Either render a Provider or set a defaultValue for this Store.`, s);
    }
    return state;
}
exports.useStore = useStore;
function useStoreOptionally(s, deps) {
    try {
        const store = useStore(s, deps);
        return [store, true];
    }
    catch (e) {
        if (e instanceof error_1.NoStoreError) {
            return [undefined, false];
        }
        else {
            throw e;
        }
    }
}
exports.useStoreOptionally = useStoreOptionally;
function compare(oldDeps, newDeps) {
    if (oldDeps.length !== newDeps.length) {
        return true;
    }
    for (const index in newDeps) {
        if (oldDeps[index] !== newDeps[index]) {
            return true;
        }
    }
    return false;
}
